﻿// 
// This is the source code of Telegram for Windows Phone v. 3.x.x.
// It is licensed under GNU GPL v. 2 or later.
// You should have received a copy of the license in this archive (see LICENSE).
// 
// Copyright Evgeny Nadymov, 2013-present.
// 
using System;
using System.Diagnostics;
using Telegram.Api.Extensions;
using Telegram.Api.TL;
using Telegram.Api.TL.Functions.Stuff;

namespace Telegram.Api.Services
{
    public partial class MTProtoService
    {
        private object _debugRoot = new object();

        public void MessageAcknowledgments(TLVector<TLLong> ids)
        {
            PrintCaption("msgs_ack");
            TLUtils.WriteLine("ids");
            foreach (var id in ids)
            {
                TLUtils.WriteLine(TLUtils.MessageIdString(id));
            }
            var obj = new TLMessageAcknowledgments { MsgIds = ids };

            var authKey = _activeTransport.AuthKey;
            var sesseionId = _activeTransport.SessionId;
            var salt = _activeTransport.Salt;

            int sequenceNumber;
            TLLong messageId;
            lock (_activeTransportRoot)
            {
                sequenceNumber = _activeTransport.SequenceNumber * 2;
                messageId = _activeTransport.GenerateMessageId(true);
            }
            var transportMessage = CreateTLTransportMessage(salt, sesseionId, new TLInt(sequenceNumber), messageId, obj);
            var encryptedMessage = CreateTLEncryptedMessage(authKey, transportMessage);

            lock (_activeTransportRoot)
            {
                if (_activeTransport.Closed)
                {
                    _activeTransport = GetTransport(_activeTransport.Host, _activeTransport.Port, Type,
                        new TransportSettings
                        {
                            DcId = _activeTransport.DCId,
                            Secret = _activeTransport.Secret,
                            AuthKey = _activeTransport.AuthKey,
                            Salt = _activeTransport.Salt,
                            SessionId = _activeTransport.SessionId,
                            MessageIdDict = _activeTransport.MessageIdDict,
                            SequenceNumber = _activeTransport.SequenceNumber,
                            ClientTicksDelta = _activeTransport.ClientTicksDelta,
                            PacketReceivedHandler = OnPacketReceived
                        });
                }
            }

            lock (_debugRoot)
            {
                //Debug.WriteLine(">>{0, -30} MsgId {1} SeqNo {2, -4} SessionId {3}\nids:", "msgs_ack", transportMessage.MessageId.Value, transportMessage.SeqNo.Value, transportMessage.SessionId.Value);
                foreach (var id in ids)
                {
                    //Debug.WriteLine(id.Value);
                }
            }

            var captionString = string.Format("msgs_ack {0}", transportMessage.MessageId);
            SendPacketAsync(_activeTransport, captionString, encryptedMessage,
                result =>
                {
                    //Debug.WriteLine("@msgs_ack {0} result {1}", transportMessage.MessageId, result);
                },
                error =>
                {
                    //Debug.WriteLine("<<msgs_ack failed " + transportMessage.MessageId);
                });
        }


        public void PingAsync(TLLong pingId, Action<TLPong> callback, Action<TLRPCError> faultCallback = null)
        {
            var obj = new TLPing { PingId = pingId };

            SendNonInformativeMessage<TLPong>("ping", obj,
                result =>
                {
                    callback.SafeInvoke(result);
                },
                faultCallback.SafeInvoke);
        }

        public void PingDelayDisconnectAsync(TLLong pingId, TLInt disconnectDelay, Action<TLPong> callback, Action<TLRPCError> faultCallback = null)
        {
            var obj = new TLPingDelayDisconnect { PingId = pingId, DisconnectDelay = disconnectDelay };

            SendNonInformativeMessage<TLPong>("ping_delay_disconnect", obj,
                result =>
                {
                    callback.SafeInvoke(result);
                },
                faultCallback.SafeInvoke);
        }

        public void HttpWaitAsync(TLInt maxDelay, TLInt waitAfter, TLInt maxWait, Action callback, Action faultCallback)
        {
            PrintCaption("http_wait");

            var obj = new TLHttpWait { MaxDelay = maxDelay, WaitAfter = waitAfter, MaxWait = maxWait };

            var authKey = _activeTransport.AuthKey;
            var salt = _activeTransport.Salt;
            var sessionId = _activeTransport.SessionId;

            int sequenceNumber;
            TLLong messageId;
            lock (_activeTransportRoot)
            {
                sequenceNumber = _activeTransport.SequenceNumber * 2;
                messageId = _activeTransport.GenerateMessageId(true);
            }
            var transportMessage = CreateTLTransportMessage(salt, sessionId, new TLInt(sequenceNumber), messageId, obj);
            var encryptedMessage = CreateTLEncryptedMessage(authKey, transportMessage);

            lock (_activeTransportRoot)
            {
                if (_activeTransport.Closed)
                {
                    _activeTransport = GetTransport(_activeTransport.Host, _activeTransport.Port, Type,
                        new TransportSettings
                        {
                            DcId = _activeTransport.DCId,
                            Secret = _activeTransport.Secret,
                            AuthKey = _activeTransport.AuthKey,
                            Salt = _activeTransport.Salt,
                            SessionId = _activeTransport.SessionId,
                            MessageIdDict = _activeTransport.MessageIdDict,
                            SequenceNumber = _activeTransport.SequenceNumber,
                            ClientTicksDelta = _activeTransport.ClientTicksDelta,
                            PacketReceivedHandler = OnPacketReceived
                        });
                }
            }

            SendPacketAsync(_activeTransport, "http_wait " + transportMessage.MessageId, encryptedMessage,
                result =>
                {
                    //try
                    //{
                    //    ReceiveBytesAsync(result, authKey);
                    //}
                    //catch (Exception e)
                    //{
                    //    TLUtils.WriteException(e);
                    //}
                    //finally
                    //{
                    //    callback();
                    //}
                },
                error => faultCallback.SafeInvoke());
        }
    }
}
